
; memexp.asm

org 100h

        jmp     _start
;---------------------------------------------------------
msg1:
        db      31 dup (" "),"Memory Explorer",0xd,0xa
        db      30 dup (" ")
        db      18 dup ("-"),0xd,0xa,0xd,0xa
        db      28 dup (" "),"Press 'Escape' to quit",0xd,0xa,0xd,0xa
        db      15 dup (" "),"Use Up/Down Arrow keys and PageUp/Down "
        db      "to navigate",0
msg:
        rb      0x18
align 16
pmsg:
        dw      0
pos:
        dw      0
curs:
        dw      0
;---------------------------------------------------------

_start:
        mov     word [pos], 0
        call    cls
        mov     bx, msg1
        call    write_msg
        call    color
        mov     bx, 0x10
        xor     ax, ax
        mov     es, ax
        jmp     loop_

loop0:
        mov     ax, [pos]
        add     ax, 0x10
        mov     word [pos], ax
loop3:
        mov     ax, [pos]
        mov     es, ax
        mov     bx, 0x10
        jmp     loop_
loop2:
        mov     ax, es
        inc     ax
        mov     es, ax
loop_:
        push    bx
        mov     [pmsg], ax
        push    es
        mov     ax, cs
        mov     es, ax
        mov     si, pmsg
        call    print_hex
        pop     es
        mov     ah, 0xe
        mov     al, ':'
        int     0x10
        call    space
        mov     si, 0
        pop     bx
        mov     cx, 0x8
loop1:
        push    bx
        call    print_hex
        pop     bx
        dec     cx
        cmp     cx, 4
        jnz     miss
        call    space
miss:
        mov     dl, [es:si]
        mov     [msg+si], dl
        inc     si
        mov     dl, [es:si]
        mov     [msg+si], dl
        inc     si
        cmp     cx, 0
        jnz      loop1
        push    bx
        inc     di
        mov     byte [msg+si], 0
        call    print_ascii
        call    newline
        pop     bx
        dec     bx
        cmp     bx, 0
        jnz     loop2
        call    getkey
        call    setcurs
        jmp     loop3
;-----------------------------------------------------------------------

cls:
        mov     ax, 0x3
        int     0x10
        ret

delay:
        mov     cx, 0x1f
        mov     dx, 0x4240
        mov     ah, 0x86
        int     0x15
        ret

setcurs:
        mov     ah, 2
        mov     bx, 0
        xor     dx, dx
        int     0x10    ; mov cursor to screen 0, 0
        mov     ah, 1
        mov     cx, 0x2607
        int     0x10    ; make cursor invisible
        ret
;---------------------------------------------------------------------
getkey:
        mov     ah, 0x1
        int     0x16
        jnz     gotit
        ret
gotit:
        mov     ah, 0x0
        int     0x16    ; clears keyboard buffer
        push    ax
        call    setcurs
        pop     ax
        cmp     ax, 0x011b
        jz      exit
        cmp     ax, 0x4800
        jz      up
        cmp     ax, 0x5000
        jz      down
        cmp     ax, 0x4900
        jz      pup
        cmp     ax, 0x5100
        jz      pdown
        ret

pup:
        mov     ax, [pos]
        sub     ax, 0x110
        mov     word [pos], ax
        jmp     down
pdown:
        mov     ax, [pos]
        add     ax, 0xf0
        mov     word [pos], ax
        jmp     down

up:     mov     ax, [pos]
        sub     ax, 0x20
        mov     word [pos], ax
down:
        call    setcurs
        pop     bx      ; remove return addr.
        jmp     loop0
exit:
        call    cls
        mov     ah, 1
        mov     cx, 0x0607
        int     0x10    ; restore cursor
        mov     ah, 0x4c
        mov     al, 0
        int     0x21
;---------------------------------------------------------------------

write_msg:
        push    bx
        mov     ah, 2
        mov     dx, 0x1100
        mov     bx, 0
        int     0x10
        pop     bx
wbeg:
        mov     al, [bx]
        test    al, al
        jz      wend
        push    bx
        mov     ah, 0x0e
        int     0x10
        pop     bx
        inc     bx
        jmp     wbeg
wend:
        call    setcurs
        ret
;--------------------------------------------------------        

print_hex:
        push    cx
        mov     cx, 0x4         ; 4 hex digits
        mov     dx, [es:si]     ; load word into dx
print_digit:
        rol     dx, byte 0x4    ; rotate so that lowest 4 bits are used
        mov     ah, 0xe 
        mov     al, dl          ; mask off so we have only next nibble
        and     al, 0xf
        add     al, 0x30        ; convert to 0-based digit
        cmp     al, 0x39        ; check for overflow
        jbe     good_digit
        add     al, 0x7
good_digit:
        int     0x10
        loop    print_digit
        call    space
        pop     cx
        ret
;-------------------------
print_ascii:
        mov     ah, 0xe
        call    space
        call    space
        mov     cx, 0x10
        mov     si, msg
abeg:
        mov     al, [si]
        cmp     al, 0x1f
        jle     non_ascii
        cmp     al, 0x7f
        jl      short ascii
non_ascii:
        mov     al, 0x2e
ascii:
        int     0x10
        inc     si
        loop    abeg
        ret
;--------------------------
space:
        mov     ah, 0xe
        mov     al, ' '
        int     0x10
        ret

newline:
        mov     ah, 0x0e
        mov     al, 0x0d
        int     0x10
        mov     al, 0x0a
        int     0x10
        ret
;--------------------------
color:
        mov     si, 1
        mov     cx, 0x500
        mov     ax, 0xb8a0
        mov     fs, ax
scan:
        mov     byte [fs:si], 0x1b
        inc     si
        inc     si
        loop    scan
        ret
;-----------------------------------------------------------
;EOF
